from rest_framework import serializers
from arkindex.dataimport.models import Worker, WorkerVersion
from arkindex.project.serializer_fields import EnumField
from arkindex_common.ml_tool import MLToolType
from arkindex.dataimport.serializers.git import RevisionWithRefsSerializer


class WorkerSerializer(serializers.ModelSerializer):
    """
    Serialize a repository worker
    """
    type = EnumField(MLToolType)

    class Meta:
        model = Worker
        fields = (
            'id',
            'name',
            'slug',
            'type',
        )


class WorkerVersionSerializer(serializers.ModelSerializer):
    """
    Serialize a worker version
    """
    class Meta:
        model = WorkerVersion
        fields = (
            'id',
            'configuration',
            'revision',
            'docker_image',
        )

    def to_representation(self, instance):
        self.fields['revision'] = RevisionWithRefsSerializer(read_only=True)
        return super(WorkerVersionSerializer, self).to_representation(instance)
