# Generated by Django 2.2.9 on 2020-01-17 15:39

import arkindex.project.aws
import arkindex.project.fields
import arkindex.project.polygon
from django.db import migrations, models
import django.db.models.deletion
import enumfields.fields
import uuid

POLYGON_HASH_FORWARD_SQL = """
ALTER TABLE images_zone
    ADD COLUMN IF NOT EXISTS polygon_hash VARCHAR(32)
    GENERATED ALWAYS AS (md5(polygon::varchar)) STORED;
CREATE UNIQUE INDEX zone_unique_image_polygon ON images_zone (image_id, polygon_hash);
"""

POLYGON_HASH_REVERSE_SQL = """
DROP INDEX IF EXISTS zone_unique_image_polygon;
ALTER TABLE images_zone DROP COLUMN IF EXISTS polygon_hash;
"""


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('dataimport', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Image',
            fields=[
                ('status', enumfields.fields.EnumField(default='unchecked', enum=arkindex.project.aws.S3FileStatus, max_length=50)),
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('path', arkindex.project.fields.LStripTextField(chars='/')),
                ('width', models.PositiveIntegerField(default=0)),
                ('height', models.PositiveIntegerField(default=0)),
                ('hash', arkindex.project.fields.MD5HashField(blank=True, max_length=32, null=True)),
                ('datafile', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='images', to='dataimport.DataFile')),
            ],
            bases=(arkindex.project.aws.S3FileMixin, models.Model),
        ),
        migrations.CreateModel(
            name='ImageServer',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('display_name', models.CharField(max_length=250)),
                ('url', arkindex.project.fields.StripSlashURLField(unique=True)),
                ('s3_bucket', models.SlugField(blank=True, db_index=False, max_length=63, null=True)),
                ('s3_region', models.SlugField(blank=True, db_index=False, max_length=63, null=True)),
                ('max_width', models.PositiveIntegerField(blank=True, null=True)),
                ('max_height', models.PositiveIntegerField(blank=True, null=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('validated', models.BooleanField(default=False)),
                ('read_only', models.BooleanField(default=False)),
            ],
        ),
        migrations.CreateModel(
            name='Zone',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('polygon', arkindex.project.polygon.PolygonField()),
                ('image', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='zones', to='images.Image')),
            ],
        ),
        migrations.AddField(
            model_name='image',
            name='server',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='images', to='images.ImageServer'),
        ),
        # When resetting migrations, replace the default AddConstraint on zone_unique_image_polygon with this RunSQL
        # Required to add the unique constraint on the polygon_hash field, which is invisible to Django
        # Otherwise, you will run into errors related to the requirement of an operator class for polygon on btree indexes.
        migrations.RunSQL(
            sql=POLYGON_HASH_FORWARD_SQL,
            reverse_sql=POLYGON_HASH_REVERSE_SQL,
            state_operations=[
                migrations.AddConstraint(
                    model_name='zone',
                    constraint=models.UniqueConstraint(
                        fields=('image', 'polygon'),
                        name='zone_unique_image_polygon',
                    ),
                ),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='image',
            unique_together={('server', 'path'), ('server', 'hash')},
        ),
    ]
